'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _bluebird;

function _load_bluebird() {
  return _bluebird = require('bluebird');
}

var getComponentStatus = function () {
  var _ref5 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(consumer, component, checkoutProps) {
    var version, latestVersion, reset, componentModel, componentStatus, returnFailure, getNewVersion, newVersion, existingBitMapId, currentlyUsedVersion, baseComponentVersion, baseComponent, isModified, mergeResults, newBitId, currentComponent, versionRef, componentVersion, newId;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            version = checkoutProps.version, latestVersion = checkoutProps.latestVersion, reset = checkoutProps.reset;
            _context3.next = 3;
            return consumer.scope.getModelComponentIfExist(component.id);

          case 3:
            componentModel = _context3.sent;
            componentStatus = { id: component.id };

            returnFailure = function returnFailure(msg) {
              componentStatus.failureMessage = msg;
              return componentStatus;
            };

            if (componentModel) {
              _context3.next = 8;
              break;
            }

            return _context3.abrupt('return', returnFailure(`component ${component.id.toString()} doesn't have any version yet`));

          case 8:
            getNewVersion = function getNewVersion() {
              if (reset) return component.id.version;
              // $FlowFixMe if !reset the version is defined
              return latestVersion ? componentModel.latest() : version;
            };

            newVersion = getNewVersion();

            if (!(version && !latestVersion && !componentModel.hasVersion(version))) {
              _context3.next = 12;
              break;
            }

            return _context3.abrupt('return', returnFailure(`component ${component.id.toStringWithoutVersion()} doesn't have version ${version}`));

          case 12:
            existingBitMapId = consumer.bitMap.getBitId(component.id, { ignoreVersion: true });
            currentlyUsedVersion = existingBitMapId.version;

            if (!(version && currentlyUsedVersion === version)) {
              _context3.next = 16;
              break;
            }

            return _context3.abrupt('return', returnFailure(`component ${component.id.toStringWithoutVersion()} is already at version ${version}`));

          case 16:
            if (!(latestVersion && currentlyUsedVersion === newVersion)) {
              _context3.next = 18;
              break;
            }

            return _context3.abrupt('return', returnFailure(`component ${component.id.toStringWithoutVersion()} is already at the latest version, which is ${newVersion}`));

          case 18:
            _context3.next = 20;
            return componentModel.loadVersion(currentlyUsedVersion, consumer.scope.objects);

          case 20:
            baseComponentVersion = _context3.sent;
            _context3.next = 23;
            return consumer.loadComponentFromModel(existingBitMapId);

          case 23:
            baseComponent = _context3.sent;
            _context3.next = 26;
            return consumer.isComponentModified(baseComponentVersion, component);

          case 26:
            isModified = _context3.sent;

            if (!(!isModified && reset)) {
              _context3.next = 29;
              break;
            }

            return _context3.abrupt('return', returnFailure(`component ${component.id.toStringWithoutVersion()} is not modified`));

          case 29:
            mergeResults = void 0;

            if (!(isModified && version)) {
              _context3.next = 38;
              break;
            }

            newBitId = component.id.changeVersion(newVersion);
            _context3.next = 34;
            return consumer.loadComponentFromModel(newBitId);

          case 34:
            currentComponent = _context3.sent;
            _context3.next = 37;
            return (0, (_mergeVersion || _load_mergeVersion()).threeWayMerge)({
              consumer,
              otherComponent: component,
              otherVersion: currentlyUsedVersion,
              currentComponent,
              currentVersion: newVersion,
              baseComponent
            });

          case 37:
            mergeResults = _context3.sent;

          case 38:
            versionRef = componentModel.versions[newVersion];
            _context3.next = 41;
            return consumer.scope.getObject(versionRef.hash);

          case 41:
            componentVersion = _context3.sent;
            newId = component.id.changeVersion(newVersion);
            return _context3.abrupt('return', { componentFromFS: component, componentFromModel: componentVersion, id: newId, mergeResults });

          case 44:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function getComponentStatus(_x3, _x4, _x5) {
    return _ref5.apply(this, arguments);
  };
}();

/**
 * 1) when the files are modified with conflicts and the strategy is "ours", leave the FS as is
 * and update only bitmap id version. (not the componentMap object).
 *
 * 2) when the files are modified with conflicts and the strategy is "theirs", write the component
 * according to id.version.
 *
 * 3) when files are modified with no conflict or files are modified with conflicts and the
 * strategy is manual, load the component according to id.version and update component.files.
 * applyModifiedVersion() docs explains what files are updated/added.
 *
 * 4) when --reset flag is used, write the component according to the bitmap version
 *
 * Side note:
 * Deleted file => if files are in used version but not in the modified one, no need to delete it. (similar to git).
 * Added file => if files are not in used version but in the modified one, they'll be under mergeResults.addFiles
 */


var applyVersion = function () {
  var _ref6 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee5(consumer, id, componentFromFS, mergeResults, checkoutProps) {
    var _this = this;

    var mergeStrategy, verbose, skipNpmInstall, ignoreDist, filesStatus, componentWithDependencies, componentMap, rootDir, shouldWritePackageJson, shouldInstallNpmPackages, writePackageJson, files, modifiedStatus, shouldDependenciesSaveAsComponents, manyComponentsWriter;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            mergeStrategy = checkoutProps.mergeStrategy, verbose = checkoutProps.verbose, skipNpmInstall = checkoutProps.skipNpmInstall, ignoreDist = checkoutProps.ignoreDist;
            filesStatus = {};

            if (!(mergeResults && mergeResults.hasConflicts && mergeStrategy === (_mergeVersion || _load_mergeVersion()).MergeOptions.ours)) {
              _context5.next = 6;
              break;
            }

            componentFromFS.files.forEach(function (file) {
              filesStatus[(0, (_path2 || _load_path2()).pathNormalizeToLinux)(file.relative)] = (_mergeVersion || _load_mergeVersion()).FileStatus.unchanged;
            });
            consumer.bitMap.updateComponentId(id);
            return _context5.abrupt('return', { id, filesStatus });

          case 6:
            _context5.next = 8;
            return consumer.loadComponentWithDependenciesFromModel(id);

          case 8:
            componentWithDependencies = _context5.sent;
            componentMap = componentFromFS.componentMap;

            if (componentMap) {
              _context5.next = 12;
              break;
            }

            throw new (_generalError || _load_generalError()).default('applyVersion: componentMap was not found');

          case 12:
            if (componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED && !id.scope) {
              componentWithDependencies.dependencies = [];
              componentWithDependencies.devDependencies = [];
              componentWithDependencies.compilerDependencies = [];
              componentWithDependencies.testerDependencies = [];
              componentWithDependencies.allDependencies = [];
            }
            rootDir = componentMap.rootDir;

            shouldWritePackageJson = function () {
              var _ref7 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4() {
                var packageJsonPath;
                return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
                  while (1) {
                    switch (_context4.prev = _context4.next) {
                      case 0:
                        if (rootDir) {
                          _context4.next = 2;
                          break;
                        }

                        return _context4.abrupt('return', false);

                      case 2:
                        packageJsonPath = (_path || _load_path()).default.join(consumer.getPath(), rootDir, 'package.json');
                        return _context4.abrupt('return', (_fsExtra || _load_fsExtra()).default.exists(packageJsonPath));

                      case 4:
                      case 'end':
                        return _context4.stop();
                    }
                  }
                }, _callee4, _this);
              }));

              return function shouldWritePackageJson() {
                return _ref7.apply(this, arguments);
              };
            }();

            shouldInstallNpmPackages = function shouldInstallNpmPackages() {
              if (componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) return false;
              return !skipNpmInstall;
            };

            _context5.next = 18;
            return shouldWritePackageJson();

          case 18:
            writePackageJson = _context5.sent;
            files = componentWithDependencies.component.files;

            files.forEach(function (file) {
              filesStatus[(0, (_path2 || _load_path2()).pathNormalizeToLinux)(file.relative)] = (_mergeVersion || _load_mergeVersion()).FileStatus.updated;
            });

            modifiedStatus = {};

            if (mergeResults) {
              // update files according to the merge results
              modifiedStatus = applyModifiedVersion(files, mergeResults, mergeStrategy);
            }
            _context5.next = 25;
            return consumer.shouldDependenciesSavedAsComponents([id]);

          case 25:
            shouldDependenciesSaveAsComponents = _context5.sent;

            componentWithDependencies.component.dependenciesSavedAsComponents = shouldDependenciesSaveAsComponents[0].saveDependenciesAsComponents;

            manyComponentsWriter = new (_manyComponentsWriter || _load_manyComponentsWriter()).default({
              consumer,
              componentsWithDependencies: [componentWithDependencies],
              installNpmPackages: shouldInstallNpmPackages(),
              override: true,
              writeConfig: Boolean(componentMap.configDir), // write bit.json and config files only if it was there before
              configDir: componentMap.configDir,
              verbose,
              writeDists: !ignoreDist,
              writePackageJson
            });
            _context5.next = 30;
            return manyComponentsWriter.writeAll();

          case 30:
            return _context5.abrupt('return', { id, filesStatus: Object.assign(filesStatus, modifiedStatus) });

          case 31:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function applyVersion(_x6, _x7, _x8, _x9, _x10) {
    return _ref6.apply(this, arguments);
  };
}();

/**
 * relevant only when
 * 1) there is no conflict => add files from mergeResults: addFiles, overrideFiles and modifiedFiles.output.
 * 2) there is conflict and mergeStrategy is manual => add files from mergeResults: addFiles, overrideFiles and modifiedFiles.conflict.
 *
 * this function only updates the files content, it doesn't write the files
 */


exports.applyModifiedVersion = applyModifiedVersion;

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _pMapSeries;

function _load_pMapSeries() {
  return _pMapSeries = _interopRequireDefault(require('p-map-series'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _;

function _load_() {
  return _ = require('..');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../component'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _path2;

function _load_path2() {
  return _path2 = require('../../utils/path');
}

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('../../scope/models/version'));
}

var _sources;

function _load_sources() {
  return _sources = require('../component/sources');
}

var _mergeVersion;

function _load_mergeVersion() {
  return _mergeVersion = require('./merge-version');
}

var _generalError;

function _load_generalError() {
  return _generalError = _interopRequireDefault(require('../../error/general-error'));
}

var _manyComponentsWriter;

function _load_manyComponentsWriter() {
  return _manyComponentsWriter = _interopRequireDefault(require('../component-ops/many-components-writer'));
}

var _repositories;

function _load_repositories() {
  return _repositories = require('../../scope/repositories');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function () {
  var _ref = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(consumer, checkoutProps) {
    var getAllComponentsStatus = function () {
      var _ref4 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee() {
        var tmp, componentsStatus;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                tmp = new (_repositories || _load_repositories()).Tmp(consumer.scope);
                _context.prev = 1;
                _context.next = 4;
                return Promise.all(components.map(function (component) {
                  return getComponentStatus(consumer, component, checkoutProps);
                }));

              case 4:
                componentsStatus = _context.sent;
                _context.next = 7;
                return tmp.clear();

              case 7:
                return _context.abrupt('return', componentsStatus);

              case 10:
                _context.prev = 10;
                _context.t0 = _context['catch'](1);
                _context.next = 14;
                return tmp.clear();

              case 14:
                throw _context.t0;

              case 15:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[1, 10]]);
      }));

      return function getAllComponentsStatus() {
        return _ref4.apply(this, arguments);
      };
    }();

    var version, ids, promptMergeOptions, _ref2, components, allComponentsStatus, componentWithConflict, failedComponents, succeededComponents, componentsResults;

    return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            version = checkoutProps.version, ids = checkoutProps.ids, promptMergeOptions = checkoutProps.promptMergeOptions;
            _context2.next = 3;
            return consumer.loadComponents(ids);

          case 3:
            _ref2 = _context2.sent;
            components = _ref2.components;
            _context2.next = 7;
            return getAllComponentsStatus();

          case 7:
            allComponentsStatus = _context2.sent;
            componentWithConflict = allComponentsStatus.find(function (component) {
              return component.mergeResults && component.mergeResults.hasConflicts;
            });

            if (!componentWithConflict) {
              _context2.next = 16;
              break;
            }

            if (!(!promptMergeOptions && !checkoutProps.mergeStrategy)) {
              _context2.next = 12;
              break;
            }

            throw new (_generalError || _load_generalError()).default(`automatic merge has failed for component ${componentWithConflict.id.toStringWithoutVersion()}.\nplease use "--manual" to manually merge changes or use "--theirs / --ours" to choose one of the conflicted versions`);

          case 12:
            if (checkoutProps.mergeStrategy) {
              _context2.next = 16;
              break;
            }

            _context2.next = 15;
            return (0, (_mergeVersion || _load_mergeVersion()).getMergeStrategyInteractive)();

          case 15:
            checkoutProps.mergeStrategy = _context2.sent;

          case 16:
            failedComponents = allComponentsStatus.filter(function (componentStatus) {
              return componentStatus.failureMessage;
            }) // $FlowFixMe componentStatus.failureMessage is set
            .map(function (componentStatus) {
              return { id: componentStatus.id, failureMessage: componentStatus.failureMessage };
            });
            succeededComponents = allComponentsStatus.filter(function (componentStatus) {
              return !componentStatus.failureMessage;
            });
            // do not use Promise.all for applyVersion. otherwise, it'll write all components in parallel,
            // which can be an issue when some components are also dependencies of others

            _context2.next = 20;
            return (0, (_pMapSeries || _load_pMapSeries()).default)(succeededComponents, function (_ref3) {
              var id = _ref3.id,
                  componentFromFS = _ref3.componentFromFS,
                  mergeResults = _ref3.mergeResults;

              return applyVersion(consumer, id, componentFromFS, mergeResults, checkoutProps);
            });

          case 20:
            componentsResults = _context2.sent;
            return _context2.abrupt('return', { components: componentsResults, version, failedComponents });

          case 22:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  function checkoutVersion(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return checkoutVersion;
}();

function applyModifiedVersion(componentFiles, mergeResults, mergeStrategy) {
  var filesStatus = {};
  if (mergeResults.hasConflicts && mergeStrategy !== (_mergeVersion || _load_mergeVersion()).MergeOptions.manual) return filesStatus;
  mergeResults.modifiedFiles.forEach(function (file) {
    var filePath = (_path || _load_path()).default.normalize(file.filePath);
    var foundFile = componentFiles.find(function (componentFile) {
      return componentFile.relative === filePath;
    });
    if (!foundFile) throw new (_generalError || _load_generalError()).default(`file ${filePath} not found`);
    if (file.conflict) {
      foundFile.contents = Buffer.from(file.conflict);
      filesStatus[file.filePath] = (_mergeVersion || _load_mergeVersion()).FileStatus.manual;
    } else if (file.output) {
      foundFile.contents = Buffer.from(file.output);
      filesStatus[file.filePath] = (_mergeVersion || _load_mergeVersion()).FileStatus.merged;
    } else {
      throw new (_generalError || _load_generalError()).default('file does not have output nor conflict');
    }
  });
  mergeResults.addFiles.forEach(function (file) {
    componentFiles.push(file.fsFile);
    filesStatus[file.filePath] = (_mergeVersion || _load_mergeVersion()).FileStatus.added;
  });
  mergeResults.overrideFiles.forEach(function (file) {
    var filePath = (_path || _load_path()).default.normalize(file.filePath);
    var foundFile = componentFiles.find(function (componentFile) {
      return componentFile.relative === filePath;
    });
    if (!foundFile) throw new (_generalError || _load_generalError()).default(`file ${filePath} not found`);
    foundFile.contents = file.fsFile.contents;
    filesStatus[file.filePath] = (_mergeVersion || _load_mergeVersion()).FileStatus.overridden;
  });

  return filesStatus;
}