'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _bluebird;

function _load_bluebird() {
  return _bluebird = require('bluebird');
}

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _ajv;

function _load_ajv() {
  return _ajv = _interopRequireDefault(require('ajv'));
}

var _semver;

function _load_semver() {
  return _semver = _interopRequireDefault(require('semver'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

var _logger2;

function _load_logger2() {
  return _logger2 = require('../logger/logger');
}

var _scope;

function _load_scope() {
  return _scope = require('../scope');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _extensionNameNotValid;

function _load_extensionNameNotValid() {
  return _extensionNameNotValid = _interopRequireDefault(require('./exceptions/extension-name-not-valid'));
}

var _extensionGetDynamicConfigError;

function _load_extensionGetDynamicConfigError() {
  return _extensionGetDynamicConfigError = _interopRequireDefault(require('./exceptions/extension-get-dynamic-config-error'));
}

var _analytics;

function _load_analytics() {
  return _analytics = require('../analytics/analytics');
}

var _extensionLoadError;

function _load_extensionLoadError() {
  return _extensionLoadError = _interopRequireDefault(require('./exceptions/extension-load-error'));
}

var _environment;

function _load_environment() {
  return _environment = _interopRequireDefault(require('../environment'));
}

var _extensionSchemaError;

function _load_extensionSchemaError() {
  return _extensionSchemaError = _interopRequireDefault(require('./exceptions/extension-schema-error'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ajv = new (_ajv || _load_ajv()).default();

var CORE_EXTENSIONS_PATH = './core-extensions';

// export type BaseExtensionProps = {
//   ...InstanceSpecificProps,
//   ...StaticProps
// };

// type staticProps = $Diff<BaseExtensionProps, instanceSpecificProps>

var BaseExtension = function () {
  // Store the required plugin
  function BaseExtension(extensionProps) {
    _classCallCheck(this, BaseExtension);

    this.api = _getConcreteBaseAPI({ name: this.name });

    this.name = extensionProps.name;
    this.rawConfig = extensionProps.rawConfig;
    this.schema = extensionProps.schema;
    this.options = extensionProps.options;
    this.dynamicConfig = extensionProps.dynamicConfig || extensionProps.rawConfig;
    this.context = extensionProps.context;
    this.script = extensionProps.script;
    this.disabled = extensionProps.disabled;
    this.filePath = extensionProps.filePath;
    this.rootDir = extensionProps.rootDir || '';
    this.loaded = extensionProps.loaded;
    this.api = extensionProps.api;
  } // Store the required plugin


  _createClass(BaseExtension, [{
    key: 'init',


    /**
     * Run the extension's init function
     */
    value: function () {
      var _ref = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee() {
        var throws = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
        var initOptions;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'initialize extension');
                _context.prev = 1;
                initOptions = {};

                if (this.script && this.script.init && typeof this.script.init === 'function') {
                  initOptions = this.script.init({
                    rawConfig: this.rawConfig,
                    dynamicConfig: this.dynamicConfig,
                    api: this.api
                  });
                }
                // wrap in promise, in case a script has async init
                _context.next = 6;
                return Promise.resolve(initOptions);

              case 6:
                this.initOptions = _context.sent;

                this.initialized = true;
                // Make sure to not kill the process if an extension didn't load correctly
                _context.next = 18;
                break;

              case 10:
                _context.prev = 10;
                _context.t0 = _context['catch'](1);

                (_logger || _load_logger()).default.error(`initialized extension ${this.name} failed`);
                (_logger || _load_logger()).default.error(_context.t0);

                if (!throws) {
                  _context.next = 16;
                  break;
                }

                throw new (_extensionLoadError || _load_extensionLoadError()).default(_context.t0, this.name);

              case 16:
                this.initialized = false;
                return _context.abrupt('return', false);

              case 18:
                return _context.abrupt('return', true);

              case 19:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[1, 10]]);
      }));

      function init() {
        return _ref.apply(this, arguments);
      }

      return init;
    }()
  }, {
    key: 'extendAPI',
    value: function extendAPI(baseApi, api) {
      this.api = (_ramda || _load_ramda()).default.merge(baseApi, api);
    }
  }, {
    key: 'toString',
    value: function toString() {
      return JSON.stringify(this, null, 2);
    }
  }, {
    key: 'toBitJsonObject',
    value: function toBitJsonObject() {
      return {
        [this.name]: {
          rawConfig: this.rawConfig,
          options: this.options
        }
      };
    }
  }, {
    key: 'toModelObject',
    value: function toModelObject() {
      return {
        name: this.name,
        config: this.dynamicConfig
      };
    }
  }, {
    key: 'toObject',
    value: function toObject() {
      return this.toModelObject();
    }

    /**
     * Reload the extension, this mainly contain the process of going to the extension file requiring it and get the dynamic config
     * It mostly used for env extension when sometime on the first load the env didn't installed yet (only during build / test) phase
     */

  }, {
    key: 'reload',
    value: function () {
      var _ref2 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(scopePath, _ref3) {
        var throws = _ref3.throws;

        var _getExtensionPath2, _resolvedPath, _componentPath, baseProps;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'reload extension');
                if (!this.filePath && !this.options.core) {
                  _getExtensionPath2 = _getExtensionPath(this.name, scopePath, this.options.core), _resolvedPath = _getExtensionPath2.resolvedPath, _componentPath = _getExtensionPath2.componentPath;

                  this.filePath = _resolvedPath;
                  this.rootDir = _componentPath;
                }
                this.name = _addVersionToNameFromPathIfMissing(this.name, this.rootDir, this.options);
                _context2.next = 5;
                return BaseExtension.loadFromFile({
                  name: this.name,
                  filePath: this.filePath,
                  rootDir: this.rootDir,
                  rawConfig: this.rawConfig,
                  options: this.options,
                  throws
                });

              case 5:
                baseProps = _context2.sent;

                if (!baseProps.loaded) {
                  _context2.next = 12;
                  break;
                }

                this.loaded = baseProps.loaded;
                this.script = baseProps.script;
                this.dynamicConfig = baseProps.dynamicConfig;
                _context2.next = 12;
                return this.init();

              case 12:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function reload(_x2, _x3) {
        return _ref2.apply(this, arguments);
      }

      return reload;
    }()
  }, {
    key: 'setExtensionPathInScope',
    value: function setExtensionPathInScope(scopePath) {
      var _getExtensionPath3 = _getExtensionPath(this.name, scopePath, this.options.core),
          resolvedPath = _getExtensionPath3.resolvedPath,
          componentPath = _getExtensionPath3.componentPath;

      this.filePath = resolvedPath;
      this.rootDir = componentPath;
    }
  }, {
    key: 'writeConfigFilesOnAction',
    get: function get() {
      if (!this.initOptions) {
        return false;
      }
      return this.initOptions.writeConfigFilesOnAction;
    }
  }, {
    key: 'initOptions',
    get: function get() {
      return this._initOptions;
    },
    set: function set(opts) {
      var defaultInitOpts = {
        writeConfigFilesOnAction: false
      };
      if (!opts) {
        this._initOptions = defaultInitOpts;
        return;
      }
      var res = {};
      if (opts.write) {
        res.writeConfigFilesOnAction = true;
      }
      this._initOptions = res;
    }
  }], [{
    key: 'transformStringToModelObject',
    value: function transformStringToModelObject(name) {
      return {
        name,
        config: {}
      };
    }

    /**
     * Load extension by name
     * The extension will be from scope by default or from file
     * if there is file(path) in the options
     * The file path is relative to the bit.json of the project or absolute
     * @param {string} name - name of the extension
     * @param {Object} rawConfig - raw config for the extension
     * @param {Object} options - extension options such as - disabled, file, core
     * @param {string} consumerPath - path to the consumer folder (to load the file relatively)
     * @param {string} scopePath - scope which stores the extension code
     */

  }, {
    key: 'load',
    value: function () {
      var _ref4 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(_ref5) {
        var name = _ref5.name,
            _ref5$rawConfig = _ref5.rawConfig,
            rawConfig = _ref5$rawConfig === undefined ? {} : _ref5$rawConfig,
            _ref5$options = _ref5.options,
            options = _ref5$options === undefined ? {} : _ref5$options,
            consumerPath = _ref5.consumerPath,
            scopePath = _ref5.scopePath,
            _ref5$throws = _ref5.throws,
            throws = _ref5$throws === undefined ? false : _ref5$throws,
            context = _ref5.context;

        var concreteBaseAPI, absPath, _file, _staticExtensionProps, _extensionProps, staticExtensionProps, _getExtensionPath4, _resolvedPath2, _componentPath2, nameWithVersion, extensionProps;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                (_logger || _load_logger()).default.debug(`base-extension loading ${name}`);
                concreteBaseAPI = _getConcreteBaseAPI({ name });

                if (!options.file) {
                  _context3.next = 12;
                  break;
                }

                absPath = options.file;
                _file = options.file || '';

                if (!(_path || _load_path()).default.isAbsolute(options.file) && consumerPath) {
                  absPath = (_path || _load_path()).default.resolve(consumerPath, _file);
                }
                _context3.next = 8;
                return BaseExtension.loadFromFile({
                  name,
                  filePath: absPath,
                  rawConfig,
                  options,
                  throws
                });

              case 8:
                _staticExtensionProps = _context3.sent;
                _extensionProps = _extends({ api: concreteBaseAPI, context }, _staticExtensionProps);

                _extensionProps.api = concreteBaseAPI;
                return _context3.abrupt('return', _extensionProps);

              case 12:
                staticExtensionProps = {
                  name,
                  rawConfig,
                  dynamicConfig: rawConfig,
                  options,
                  disabled: false,
                  loaded: false,
                  filePath: ''
                };
                // Require extension from scope

                if (!scopePath) {
                  _context3.next = 19;
                  break;
                }

                _getExtensionPath4 = _getExtensionPath(name, scopePath, options.core), _resolvedPath2 = _getExtensionPath4.resolvedPath, _componentPath2 = _getExtensionPath4.componentPath;
                nameWithVersion = _addVersionToNameFromPathIfMissing(name, _componentPath2, options);
                _context3.next = 18;
                return BaseExtension.loadFromFile({
                  name: nameWithVersion,
                  filePath: _resolvedPath2,
                  rootDir: _componentPath2,
                  rawConfig,
                  options,
                  throws
                });

              case 18:
                staticExtensionProps = _context3.sent;

              case 19:
                extensionProps = _extends({ api: concreteBaseAPI, context }, staticExtensionProps);
                return _context3.abrupt('return', extensionProps);

              case 21:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function load(_x4) {
        return _ref4.apply(this, arguments);
      }

      return load;
    }()
  }, {
    key: 'loadFromModelObject',
    value: function loadFromModelObject(modelObject) {
      var staticExtensionProps = void 0;
      if (typeof modelObject === 'string') {
        staticExtensionProps = {
          name: modelObject,
          rawConfig: {},
          dynamicConfig: {},
          options: {},
          disabled: false,
          loaded: false,
          filePath: ''
        };
      } else {
        staticExtensionProps = {
          name: modelObject.name,
          rawConfig: modelObject.config,
          dynamicConfig: modelObject.config,
          options: {},
          disabled: false,
          loaded: false,
          filePath: ''
        };
      }

      var concreteBaseAPI = _getConcreteBaseAPI({ name: staticExtensionProps.name });
      var extensionProps = _extends({ api: concreteBaseAPI }, staticExtensionProps);
      return extensionProps;
    }
  }, {
    key: 'loadFromFile',
    value: function () {
      var _ref6 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(_ref7) {
        var name = _ref7.name,
            filePath = _ref7.filePath,
            rootDir = _ref7.rootDir,
            _ref7$rawConfig = _ref7.rawConfig,
            rawConfig = _ref7$rawConfig === undefined ? {} : _ref7$rawConfig,
            _ref7$options = _ref7.options,
            options = _ref7$options === undefined ? {} : _ref7$options,
            _ref7$throws = _ref7.throws,
            throws = _ref7$throws === undefined ? false : _ref7$throws;

        var extensionProps, isFileExist, err, _script, valid, msg, printStack;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                (_logger || _load_logger()).default.debug(`base-extension, loading extension ${name} from ${filePath}`);
                extensionProps = {
                  name,
                  rawConfig,
                  dynamicConfig: rawConfig,
                  options,
                  disabled: false,
                  loaded: false,
                  filePath: '',
                  rootDir: ''
                };
                // Skip disabled extensions

                if (!options.disabled) {
                  _context4.next = 7;
                  break;
                }

                extensionProps.disabled = true;
                (_logger || _load_logger()).default.info(`skip extension ${extensionProps.name} because it is disabled`);
                extensionProps.loaded = false;
                return _context4.abrupt('return', extensionProps);

              case 7:
                extensionProps.filePath = filePath;
                extensionProps.rootDir = rootDir;
                _context4.next = 11;
                return (_fsExtra || _load_fsExtra()).default.exists(filePath);

              case 11:
                isFileExist = _context4.sent;

                if (isFileExist) {
                  _context4.next = 18;
                  break;
                }

                if (!(throws && options.file)) {
                  _context4.next = 16;
                  break;
                }

                err = new Error(`the file ${filePath} not found`);
                throw new (_extensionLoadError || _load_extensionLoadError()).default(err, extensionProps.name);

              case 16:
                extensionProps.loaded = false;
                return _context4.abrupt('return', extensionProps);

              case 18:
                if (!(rootDir && !(_environment || _load_environment()).default.isEnvironmentInstalled(rootDir))) {
                  _context4.next = 21;
                  break;
                }

                extensionProps.loaded = false;
                return _context4.abrupt('return', extensionProps);

              case 21:
                _context4.prev = 21;

                // $FlowFixMe
                _script = require(filePath); // eslint-disable-line

                extensionProps.script = _script.default ? _script.default : _script;

                if (!(extensionProps.script.getSchema && typeof extensionProps.script.getSchema === 'function')) {
                  _context4.next = 31;
                  break;
                }

                _context4.next = 27;
                return Promise.resolve(extensionProps.script.getSchema());

              case 27:
                extensionProps.schema = _context4.sent;
                valid = ajv.validate(extensionProps.schema, rawConfig);

                if (valid) {
                  _context4.next = 31;
                  break;
                }

                throw new (_extensionSchemaError || _load_extensionSchemaError()).default(name, ajv.errorsText());

              case 31:
                _context4.next = 44;
                break;

              case 33:
                _context4.prev = 33;
                _context4.t0 = _context4['catch'](21);

                if (_context4.t0.code === 'MODULE_NOT_FOUND') {
                  msg = `loading extension ${extensionProps.name} failed, the file ${extensionProps.filePath} not found`;

                  (_logger || _load_logger()).default.warn(msg);
                  // console.error(msg); // eslint-disable-line no-console
                }
                (_logger || _load_logger()).default.error(`loading extension ${extensionProps.name} failed`);
                (_logger || _load_logger()).default.error(_context4.t0);
                extensionProps.loaded = false;

                if (!throws) {
                  _context4.next = 43;
                  break;
                }

                printStack = true;

                if (_context4.t0 instanceof (_extensionSchemaError || _load_extensionSchemaError()).default) {
                  printStack = false;
                }
                throw new (_extensionLoadError || _load_extensionLoadError()).default(_context4.t0, extensionProps.name, printStack);

              case 43:
                return _context4.abrupt('return', extensionProps);

              case 44:
                extensionProps.loaded = true;
                return _context4.abrupt('return', extensionProps);

              case 46:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[21, 33]]);
      }));

      function loadFromFile(_x5) {
        return _ref6.apply(this, arguments);
      }

      return loadFromFile;
    }()
  }, {
    key: 'loadDynamicConfig',
    value: function loadDynamicConfig(extensionProps) {
      (_logger || _load_logger()).default.debug('base-extension - loadDynamicConfig');
      var getDynamicConfig = (_ramda || _load_ramda()).default.path(['script', 'getDynamicConfig'], extensionProps);
      if (getDynamicConfig && typeof getDynamicConfig === 'function') {
        try {
          var _dynamicConfig = getDynamicConfig({
            rawConfig: extensionProps.rawConfig
          });
          return _dynamicConfig;
        } catch (err) {
          throw new (_extensionGetDynamicConfigError || _load_extensionGetDynamicConfigError()).default(err, extensionProps.name);
        }
      }
      return undefined;
    }
  }]);

  return BaseExtension;
}();

exports.default = BaseExtension;


var _getExtensionPath = function _getExtensionPath(name, scopePath) {
  var isCore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  if (isCore) {
    return _getCoreExtensionPath(name);
  }
  if (!scopePath) {
    throw new Error('base-extension._getExtensionPath expects to get scopePath');
  }
  return _getRegularExtensionPath(name, scopePath);
};

var _getCoreExtensionPath = function _getCoreExtensionPath(name) {
  var componentPath = (_path || _load_path()).default.join(__dirname, CORE_EXTENSIONS_PATH, name);
  return {
    resolvedPath: componentPath,
    componentPath
  };
};

var _getRegularExtensionPath = function _getRegularExtensionPath(name, scopePath) {
  var bitId = void 0;
  try {
    bitId = (_bitId || _load_bitId()).BitId.parse(name, true); // todo: make sure it always has a scope
  } catch (err) {
    throw new (_extensionNameNotValid || _load_extensionNameNotValid()).default(name);
  }
  if (!bitId || !bitId.scope) throw new (_extensionNameNotValid || _load_extensionNameNotValid()).default(name);

  var internalComponentsPath = (_scope || _load_scope()).Scope.getComponentsRelativePath();
  var internalComponentPath = (_scope || _load_scope()).Scope.getComponentRelativePath(bitId, scopePath);
  var componentPath = (_path || _load_path()).default.join(scopePath, internalComponentsPath, internalComponentPath);
  try {
    // This might throw an error in case of imported component when the env
    // isn't installed yet
    // It will be handled in higher functions
    var resolved = require.resolve(componentPath);
    return {
      resolvedPath: resolved,
      componentPath
    };
  } catch (e) {
    return {
      resolvedPath: componentPath,
      componentPath
    };
  }
};

var _getExtensionVersionFromComponentPath = function _getExtensionVersionFromComponentPath(componentPath) {
  var parsed = (_path || _load_path()).default.parse(componentPath);
  var version = parsed.base;
  if (!(_semver || _load_semver()).default.valid(version)) {
    return undefined;
  }
  return version;
};

var _addVersionToNameFromPathIfMissing = function _addVersionToNameFromPathIfMissing(name, componentPath, options) {
  if (options && options.core) return name; // if it's a core extension, it's not a bit-id.
  var bitId = void 0;
  try {
    bitId = (_bitId || _load_bitId()).BitId.parse(name, true); // @todo: make sure it always has a scope name
  } catch (err) {
    throw new (_extensionNameNotValid || _load_extensionNameNotValid()).default(name);
  }
  if (bitId.getVersion().latest) {
    var version = _getExtensionVersionFromComponentPath(componentPath);
    return bitId.changeVersion(version).toString();
  }
  return name;
};

var baseApi = {
  /**
   * API to get logger
   */
  getLogger: function getLogger(name) {
    return function () {
      return (0, (_logger2 || _load_logger2()).createExtensionLogger)(name);
    };
  }
};

/**
 * Function which get actual params and return a concrete base api
 */
var _getConcreteBaseAPI = function _getConcreteBaseAPI(_ref8) {
  var name = _ref8.name;

  var concreteBaseAPI = (_ramda || _load_ramda()).default.clone(baseApi);
  concreteBaseAPI.getLogger = baseApi.getLogger(name);
  return concreteBaseAPI;
};