'use strict';

var _chai;

function _load_chai() {
  return _chai = require('chai');
}

var _linkContent;

function _load_linkContent() {
  return _linkContent = require('./link-content');
}

describe('getLinkContent', function () {
  it('should generate content for simple js file', function () {
    var result = (0, (_linkContent || _load_linkContent()).getLinkToFileContent)('my-path.js');
    (0, (_chai || _load_chai()).expect)(result).to.equal("module.exports = require('./my-path');");
  });
  it('should generate correct content for npm vue package without .vue extension', function () {
    var result = (0, (_linkContent || _load_linkContent()).getLinkToPackageContent)('../../../src/pages/my-page.vue', '@bit/david.scope.components.vue-page');
    (0, (_chai || _load_chai()).expect)(result).to.have.string("module.exports = require('@bit/david.scope.components.vue-page');");
    (0, (_chai || _load_chai()).expect)(result).to.not.have.string("module.exports = require('@bit/david.scope.components.vue-page.vue');");
  });
  describe('es6 with link files', function () {
    var exportDefaultForm = 'exports.default =';
    var exportNonDefaultForm = 'exports.isString =';
    var importDefaultForm = "= _isString && _isString.hasOwnProperty('default') ? _isString.default : _isString;";
    var importNonDefaultForm = '= _isString.isString';
    describe('mainFile is not default and linkFile is default', function () {
      var linkContent = void 0;
      before(function () {
        var importSpecifier = [{
          mainFile: { isDefault: false, name: 'isString' },
          linkFile: { isDefault: true, name: 'isString' }
        }];
        linkContent = (0, (_linkContent || _load_linkContent()).getLinkToFileContent)('is-string.js', importSpecifier);
      });
      it('the generated link should export non-default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(exportNonDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(exportDefaultForm);
      });
      it('the generated link should import default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(importDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(importNonDefaultForm);
      });
    });
    describe('mainFile and linkFile are both default', function () {
      var linkContent = void 0;
      before(function () {
        var importSpecifier = [{
          mainFile: { isDefault: true, name: 'isString' },
          linkFile: { isDefault: true, name: 'isString' }
        }];
        linkContent = (0, (_linkContent || _load_linkContent()).getLinkToFileContent)('is-string.js', importSpecifier);
      });
      it('the generated link should export default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(exportDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(exportNonDefaultForm);
      });
      it('the generated link should import default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(importDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(importNonDefaultForm);
      });
    });
    describe('mainFile and linkFile are both non-default', function () {
      var linkContent = void 0;
      before(function () {
        var importSpecifier = [{
          mainFile: { isDefault: false, name: 'isString' },
          linkFile: { isDefault: false, name: 'isString' }
        }];
        linkContent = (0, (_linkContent || _load_linkContent()).getLinkToFileContent)('is-string.js', importSpecifier);
      });
      it('the generated link should export non-default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(exportNonDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(exportDefaultForm);
      });
      it('the generated link should import non-default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(importNonDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(importDefaultForm);
      });
    });
    describe('mainFile is default and link file is not default', function () {
      var linkContent = void 0;
      before(function () {
        var importSpecifier = [{
          mainFile: { isDefault: true, name: 'isString' },
          linkFile: { isDefault: false, name: 'isString' }
        }];
        linkContent = (0, (_linkContent || _load_linkContent()).getLinkToFileContent)('is-string.js', importSpecifier);
      });
      it('the generated link should export default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(exportDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(exportNonDefaultForm);
      });
      it('the generated link should import non-default', function () {
        (0, (_chai || _load_chai()).expect)(linkContent).to.have.string(importNonDefaultForm);
        (0, (_chai || _load_chai()).expect)(linkContent).to.not.have.string(importDefaultForm);
      });
    });
  });
});